@@ -52,8 +52,8 @@ where
                 //
                 if self
                     .data
-                    .as_ref()
-                    .and_then(|data| data.vars.get(&lhs.to_id()))
+                    .vars
+                    .get(&lhs.to_id())
                     .map(|var| var.is_fn_local && !var.declared_as_fn_param)
                     .unwrap_or(false)
                 {
@@ -75,8 +75,8 @@ where
                     //
                     if self
                         .data
-                        .as_ref()
-                        .and_then(|data| data.vars.get(&lhs.to_id()))
+                        .vars
+                        .get(&lhs.to_id())
                         .map(|var| var.is_fn_local)
                         .unwrap_or(false)
                     {
