@@ -8,6 +8,93 @@ use super::Pure;
 use crate::compress::util::{eval_as_number, is_pure_undefined_or_null};
 
 impl Pure<'_> {
+    pub(super) fn eval_free_iife(&mut self, s: &mut Stmt) {
+        fn is_fine_to_move(s: &Stmt) -> bool {
+            match s {
+                Stmt::Debugger(..) | Stmt::Empty(..) | Stmt::Expr(..) => true,
+                Stmt::Break(..)
+                | Stmt::Continue(..)
+                | Stmt::Return(..)
+                | Stmt::Decl(Decl::Var(..)) => false,
+                Stmt::Decl(..) => true,
+
+                Stmt::Block(s) => s.stmts.iter().all(is_fine_to_move),
+
+                Stmt::With(s) => is_fine_to_move(&s.body),
+                Stmt::Labeled(s) => is_fine_to_move(&s.body),
+
+                Stmt::If(s) => {
+                    is_fine_to_move(&s.cons)
+                        && s.alt.as_deref().map(is_fine_to_move).unwrap_or(true)
+                }
+                Stmt::Switch(s) => s
+                    .cases
+                    .iter()
+                    .all(|case| case.cons.iter().all(is_fine_to_move)),
+                Stmt::Throw(..) => true,
+                Stmt::Try(..) => {
+                    // TODO
+                    false
+                }
+                Stmt::While(s) => is_fine_to_move(&s.body),
+                Stmt::DoWhile(s) => is_fine_to_move(&s.body),
+                Stmt::For(..) => {
+                    // TODO
+                    false
+                }
+                Stmt::ForIn(..) => {
+                    // TODO
+                    false
+                }
+                Stmt::ForOf(..) => {
+                    // TODO
+                    false
+                }
+            }
+        }
+
+        if !self.options.unused || self.options.inline != 3 {
+            return;
+        }
+        if self.ctx.top_level {
+            return;
+        }
+
+        let es = match s {
+            Stmt::Expr(v) => v,
+            _ => return,
+        };
+
+        if let Expr::Call(CallExpr {
+            callee: Callee::Expr(callee),
+            args,
+            ..
+        }) = &mut *es.expr
+        {
+            if let Expr::Fn(FnExpr {
+                function:
+                    Function {
+                        params,
+                        body: Some(body),
+                        is_async: false,
+                        ..
+                    },
+                ..
+            }) = &mut **callee
+            {
+                if body.stmts.iter().any(|s| !is_fine_to_move(s)) {
+                    return;
+                }
+
+                if args.is_empty() && params.is_empty() {
+                    self.changed = true;
+                    tracing::debug!("Flattening iife");
+                    *s = Stmt::Block(body.take());
+                }
+            }
+        }
+    }
+
     pub(super) fn eval_array_method_call(&mut self, e: &mut Expr) {
         if !self.options.evaluate {
             return;
